<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api;

require_once 'Request.php';
require_once 'helpers.php';
require_once dirname(__FILE__).'/../autoload.php';

define('SNPAPP_LOG_TIMES', false);

use \Tools;

init();
handleRequest();

function init()
{
    header('Access-Control-Allow-Headers: content-type');
    header('Access-Control-Allow-Origin: *');
    header('Content-Type: application/json');

    $site_url = str_replace(['http://','https://'], '', _PS_BASE_URL_);

    define('APP_TOKEN', 'token_test'); // must be changed in prod !
    define('SITE_IMAGE_URL', Tools::getHttpHost(true));  // must be changed in prod !!
    define('SITE_ORDER_URL', Tools::getHttpHost(true).'/index.php?controller=order'); // must be changed in prod !
    define('SITE_URL', $site_url);
    define('SITE_PATH', __PS_BASE_URI__);
    define('ID_EMPLOYEE', 1);  // this is the id employee used by the app for backoffice admin stuff
}

function handleRequest()
{
    $post = [];
    $type = 'get';

    if (requestIsPost()) {
        $type = 'post';
        $post = getJsonFromPost();
        if (!$post) {
            return;
        }
    }

    $request = new Request($_GET, $post, $type);

    if (!checkAuth($request)) {
        respond(error("Invalid token - authentication denied."));
        return;
    }

    if (!checkMethod($request)) {
        respond(error("Missing or empty method"));
        return;
    }

    $namespace = '\\Prestalia\\App\\Api\\Handlers\\';
    $handlerClass = $request->get('method') . 'Handler';
    $handlerFile = 'handlers/' . $handlerClass . '.php';
    $handlerFull = $namespace . $handlerClass;

    if (!file_exists($handlerFile)) {
        respond(error(sprintf("No handler found for method '%s", $request->get('method'))));
        return;
    }

    include $handlerFile;

    if (!class_exists($handlerFull)) {
        respond(error(sprintf("Error loading handler for method '%s", $request->get('method'))));
        return;
    }

    if (SNPAPP_LOG_TIMES) {
        $timerStart = microtime(true);
    }

    $handler = new $handlerFull();
    respond($handler->handle($request));

    if (SNPAPP_LOG_TIMES) {
        $timerEnd = (microtime(true) - $timerStart) * 1000;
        writeLog("Handler '{$handlerClass}' took {$timerEnd} ms");
    }
}

function checkAuth(Request $request)
{
    return $request->has('token') && $request->get('token') == APP_TOKEN;
}

function checkMethod(Request $request)
{
    return $request->has('method') && trim($request->get('method')) != '';
}

function requestIsPost()
{
    return strcasecmp($_SERVER['REQUEST_METHOD'], 'post') == 0;
}

function getJsonFromPost()
{
    //Make sure that the content type of the POST request has been set to application/json
    $contentType = isset($_SERVER["CONTENT_TYPE"]) ? trim($_SERVER["CONTENT_TYPE"]) : '';
    if (strcasecmp($contentType, 'application/json') != 0) {
        respond(error('content_type_not_json'));
        return false;
    }

    $input = Tools::file_get_contents('php://input');
    $post_data = json_decode($input, true);

    if (!is_array($post_data)) {
        respond(error('invalid_json'));
        return false;
    }

    return $post_data;
}

function respond($data)
{
    echo json_encode($data, JSON_UNESCAPED_SLASHES);
}

function writeLog($message)
{
    $logfile = "log.txt";

    if (!file_exists($logfile)) {
        touch($logfile);
    }

    $f = fopen($logfile, 'a');
    fwrite($f, $message . "\n");
    fclose($f);
}
