<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Customer;
use \Order;
use \OrderState;
use \OrderDetail;
use \Db;
use \Currency;
use \Address;
use \Carrier;

class GetOrdersHandler implements Handler
{
    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_lang', 'is_numeric')) {
            return error('Invalid or missing id_lang');
        }
        if (!$request->has('id_customer', 'is_numeric')) {
            return error('Invalid or missing id_customer');
        }

        $langId = (int)$request->get('id_lang');
        $customerId = (int)$request->get('id_customer');
        $customer = new Customer($customerId);
        $orders = Order::getCustomerOrders($customer->id);

        if (!$orders) {
            return ['orders' => []];
        }

        $response = [];

        foreach ($orders as $order) {
            $response[] = $this->getFormattedOrder($order, $langId);
        }

        return ['orders' => $response];
    }

    private function getFormattedOrder(array $order, int $langId)
    {
        $orderState = new OrderState($order['current_state']);
        $orderCurrency = new Currency((int)$order['id_currency']);
        $deliveryAddress = new Address($order['id_address_delivery']);
        $invoiceAddress = new Address($order['id_address_invoice']);

        $order = [
            'id'                 => $order['reference'],
            'dateAdded'          => $order['date_add'],
            'currentState'       => $orderState->name[$langId],
            'orderStateColor'    => $orderState->color,
            'paymentMode'        => $order['payment'],
            'currency'           => $orderCurrency->iso_code,
            'priceTotal'         => (float)$order['total_paid_tax_incl'],
            'priceTotalProducts' => (float)$order['total_products_wt'],
            'priceTotalShipping' => (float)$order['total_shipping_tax_incl'],
            'discounts'          => (float)$order['total_discounts_tax_incl'],
            'products'           => $this->getProducts($order),
            'shippingInfo'       => $this->getShippingDetails($order),
            'deliveryAddress'    => $this->addressToArray($deliveryAddress),
            'invoiceAddress'     => $this->addressToArray($invoiceAddress)
        ];

        return $order;
    }

    private function getProducts(array $order)
    {
        $products = [];
        $prodDetails = OrderDetail::getList($order['id_order']);

        foreach ($prodDetails as $detail) {
            $products[] = [
                'idProduct'          => (int)$detail['product_id'],
                'idProductAttribute' => (int)$detail['product_attribute_id'],
                'name'               => $detail['product_name'],
                'reference'          => $detail['product_reference'],
                'quantity'           => (int)$detail['product_quantity'],
                'priceTotal'         => (float)$detail['total_price_tax_incl'],
                'imageUrl'           => $this->getCoverImageUrl($detail['product_id']),
            ];
        }

        return $products;
    }

    private function getShippingDetails(array $order)
    {
        $p = _DB_PREFIX_;
        $sql = "select tracking_number from {$p}order_carrier where id_order = {$order['id_order']}";
        $tracking = Db::getInstance()->getValue($sql);

        if (empty($tracking)) {
            return [];
        }

        $carrier = new Carrier($order['id_carrier']);

        return [
            'carrierName' => $carrier->name,
            'trackingCode' => $tracking
        ];
    }

    private function getCoverImageUrl(int $product_id, string $size = 'cart_'.'default')
    {
        $p = _DB_PREFIX_;
        $sql = "select id_image from {$p}image where id_product = {$product_id} and cover = 1";
        $id = Db::getInstance()->getValue($sql);
        $path = implode('/', str_split($id));
        return SITE_IMAGE_URL . "/img/p/{$path}/{$id}-{$size}.jpg";
    }

    private function addressToArray(Address $address)
    {
        return [
            'lastname'    => $address->lastname,
            'firstname'   => $address->firstname,
            'address1'    => $address->address1,
            'address2'    => $address->address2,
            'postcode'    => $address->postcode,
            'city'        => $address->city,
            'country'     => $address->country,
            'phone'       => $address->phone,
            'phoneMobile' => $address->phone_mobile
        ];
    }
}
