<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

namespace Prestalia\App\Api\Handlers;

require_once 'Handler.php';
require_once 'Request.php';
require_once 'helpers.php';

use \Prestalia\App\Api\Handler;
use \Prestalia\App\Api\Request;

use \Context;
use \Currency;
use \Customer;
use \Employee;
use \Product;

class GetProductPriceHandler implements Handler
{
    private $currencyId = null;
    private $customerId = null;

    public function __constructor()
    {
    }

    public function handle(Request $request)
    {
        if (!$request->has('id_currency', 'is_numeric')) {
            return error('invalid_id_currency');
        }

        $context = Context::getContext();
        $this->currencyId = (int)$request->get('id_currency');
        $this->customerId = $request->has('id_customer', 'is_numeric') ? (int)$request->get('id_customer') : null;

        if ($this->customerId) {
            $customer = new Customer($this->customerId);
            if ($customer->secure_key != $request->get('key')) {
                return error('invalid_key');
            }
            $context->customer = $customer;
        }

        $id_employee = ID_EMPLOYEE;
        $context->employee = new Employee($id_employee, null, $id_employee);
        $context->currency = new Currency($this->currencyId);

        try {
            if ($request->isGet()) {
                return $this->single($request);
            }

            return $this->multi($request);
        } catch (Exception $e) {
            return error('get_price_error');
        }
    }

    private function single(Request $request)
    {
        if (!$request->has('id_product', 'is_numeric')) {
            return error('invalid_id_product');
        }

        $id_product = (int)$request->get('id_product');
        $id_product_attribute =
            $request->has('id_product_attribute', 'is_numeric') ? (int)$request->get('id_product_attribute') : null;

        $result = ['prices' => $this->getPrice($id_product, $id_product_attribute)];

        if (!$result['prices']['price'] && !$result['prices']['priceOriginal']) {
            return error('invalid_data');
        }

        return $result;
    }

    private function multi(Request $request)
    {
        if (!$request->has('products', 'is_array')) {
            return error('invalid_products');
        }

        $products = array_filter(
            $request->get('products'),
            function ($e) {
                return isset($e['productId']) && $e['productId'] > 0;
            }
        );

        $products = array_map(function ($e) {
            if (!isset($e['combinationId'])) {
                $e['combinationId'] = null;
            }
            return $e;
        }, $products);

        $prices = array();
        foreach ($products as $p) {
            $prices[] = $this->getPrice($p['productId'], $p['combinationId']);
        }

        return ['prices' => $prices];
    }

    private function getPrice($productId, $combinationId)
    {
        /**
         * Product::getPriceStatic.
         *
         * @param int $id_product Product id
         * @param bool $usetax With taxes or not (optional)
         * @param int|null $id_product_attribute product attribute id (optional).
         *                                       If set to false, do not apply the combination price impact.
         *                                       NULL does apply the default combination price impact
         * @param int $decimals Number of decimals (optional)
         * @param int|null $divisor Useful when paying many time without fees (optional)
         * @param bool $only_reduc Returns only the reduction amount
         * @param bool $usereduc Set if the returned amount will include reduction
         * @param int $quantity Required for quantity discount application (default value: 1)
         * @param bool $force_associated_tax DEPRECATED - NOT USED Force to apply the associated tax.
         *                                   Only works when the parameter $usetax is true
         * @param int|null $id_customer Customer ID (for customer group reduction)
         * @param int|null $id_cart Cart ID. Required when the cookie is not accessible
         *                          (e.g., inside a payment module, a cron task...)
         * @param int|null $id_address Customer address ID. Required for price (tax included)
         *                             calculation regarding the guest localization
         * @param null $specific_price_output If a specific price applies regarding the previous parameters,
         *                                    this variable is filled with the corresponding SpecificPrice object
         * @param bool $with_ecotax insert ecotax in price output
         * @param bool $use_group_reduction
         * @param Context $context
         * @param bool $use_customer_price
         *
         * @return float Product price
         */
        $priceRetail =
        Product::getPriceStatic($productId, true, $combinationId, 2, null, false, false, 1, false, $this->customerId);
        $priceDiscount =
        Product::getPriceStatic($productId, true, $combinationId, 2, null, false, true, 1, false, $this->customerId);

        if ($priceRetail == $priceDiscount) {
            return [
                'productId' => (int)$productId,
                'combinationId' => (int)$combinationId,
                'price' => $priceRetail,
                'priceOriginal' => null
            ];
        }

        return [
            'productId' => (int)$productId,
            'combinationId' => (int)$combinationId,
            'price' => $priceDiscount,
            'priceOriginal' => $priceRetail
        ];
    }
}
