<?php
/**
*  Copyright (C) Prestalia - All Rights Reserved
*
*  Unauthorized copying of this file, via any medium is strictly prohibited
*  Proprietary and confidential
*
*  @author    Prestalia <prestalia.it>
*  @copyright 2019 - 2020 Prestalia
*  @license   Commercial
*/

require_once dirname(__FILE__).'/../autoload.php';

class SniperfastProductExporter
{
    private $idShop;
    private $idLang;
    private $config;
    private $link;
    private $idSalesCache;
    private $exportFeatures;
    private $exportAttributes;
    private $enabledIdFeatures;
    private $enabledIdAttributeGroups;

    public function __construct($idShop, $idLang, $config, $idSalesCache)
    {
        $this->idShop = $idShop;
        $this->idLang = $idLang;
        $this->config = $config;
        $this->idSalesCache = $idSalesCache;
        $this->link = new Link();

        // check if we have to export features/attributes and build enabled features/attributes array
        $this->exportFeatures = false;
        if ($config['sniper_features'] != '') {
            $this->exportFeatures = true;
            $this->enabledIdFeatures = array_map('intval', explode(',', $config['sniper_features']));
        } elseif ($config['sniper_free_ship'] > 0 || $config['sniper_export_review'] > 0) {
            $this->exportFeatures = true;
        }

        $this->exportAttributes = false;
        if ($config['sniper_attributes'] != '') {
            $this->exportAttributes = true;
            $this->enabledIdAttributeGroups = array_map('intval', explode(',', $config['sniper_attributes']));
        }

        // include file for customer functions
        if (file_exists(dirname(__FILE__).'/../customerFunctions.php')) {
            require_once dirname(__FILE__).'/../customerFunctions.php';
        }
    }

    public function exportData($idProduct, &$stats)
    {
        $data        = array();
        $skipProduct = false;

        $product = new Product($idProduct, true, $this->idLang, $this->idShop);
        if (!(bool)$product->active || $product->visibility == 'none') {
            $stats['skipped_inactive']++;
            $skipProduct = true;
        }

        if (!$skipProduct) {
            // Check if product matches price and quantity filter and skip it if it doesn't
            $productQuantity = StockAvailable::getQuantityAvailableByProduct($idProduct);

            if ($this->config['only_in_stock'] && $productQuantity < 1) {
                $stats['skipped_quantity']++;
                $skipProduct = true;
            }
        }

        if (!$skipProduct) {
            // Assign available CSV fields
            $path = '';
            $currentCategory = 0;
            $categoryPath    = '';
            $explodedPath = array();
            // $boost = array();
            if ($currentCategory != (int)$product->id_category_default) {
                if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
                    $path = Tools::getPath('', $product->id_category_default);
                } else {
                    $path = Tools::getPath($product->id_category_default);
                }
                $path = strip_tags($path);
                $path = str_replace(Configuration::get('PS_NAVIGATION_PIPE'), ' > ', $path);
                $explodedPath = explode(' > ', $path);
                if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
                    if (count($explodedPath) > 1) {
                        $explodedPath = array_slice($explodedPath, 1);
                        $path = implode(' > ', $explodedPath);
                    }
                }
                $categoryPath    = trim($path);
                $currentCategory = $product->id_category_default;
            } else {
                $path = $categoryPath;
                $explodedPath = explode(' > ', $path);
            }
            $explodedPath = array_filter(array_map('trim', $explodedPath));

            $data['id_product']         = $product->id;
            $data['id_category_default'] = $product->id_category_default;
            $data['reference']          = $product->reference;
            $data['ean13']              = $product->ean13;
            $data['pname']              = $product->name;
            $data['description']        = $product->description;
            $data['description_short']  = $product->description_short;
            $data['supplier_name']      = $product->supplier_name;
            $data['category_link_rewrite'] = Category::getLinkRewrite($product->id_category_default, $this->idLang);
            $data['plink']              = $this->link->getProductLink(
                $product->id,
                $product->link_rewrite,
                $data['category_link_rewrite'],
                $product->ean13,
                $this->idLang,
                $this->idShop
            );
            $data['mname']              = $product->manufacturer_name;
            $data['quantity']           = $productQuantity;
            $supplierInformations =
                Supplier::getProductInformationsBySupplier($product->id_supplier, $product->id, 0);
            $data['supplier_reference'] = $supplierInformations['product_supplier_reference'];
            $data['cover_image_url'] = $this->getImageLink($product, $this->link, $this->config);

            if (isset($this->idSalesCache[$product->id])) {
                $data['psales'] = (int)$this->idSalesCache[$product->id];
            } else {
                $data['psales'] = 0;
            }
            $data['category']           = implode(' ', $explodedPath);
            for ($i = 1; $i <= 4; $i++) {
                if (isset($explodedPath[$i - 1])) {
                    $data['category/lvl'.$i] = $explodedPath[$i - 1];
                } else {
                    $data['category/lvl'.$i] = '';
                }
            }

            $decimals = Configuration::get('PS_PRICE_DISPLAY_PRECISION', null, null, $this->idShop);
            if ($decimals === false) {
                $decimals = 2;
            }
            $data['price_deleted'] = (float)(round(Product::getPriceStatic(
                $product->id,
                true,
                null,
                $decimals,
                null,
                false,
                false,
                1,
                false
            ), 2));
            $data['price_deleted'] = number_format($data['price_deleted'], 2, '.', '');
            $data['price_showed'] = (float)(round(Product::getPriceStatic(
                $product->id,
                true,
                null,
                $decimals,
                null,
                false,
                true,
                1,
                false
            ), 2));
            $data['price_showed'] = number_format($data['price_showed'], 2, '.', '');
            // $data['boost_prod'] = 0;
            // $data['boost_cat']  = 0;
            // $data['boost_manu'] = 0;
            // $data['boost_supp'] = 0;
            // if (isset($boost['product'][$product->id])) {
            //     $data['boost_prod'] = $boost['product'][$product->id];
            // }
            // if (isset($boost['category'][$product->id_category_default])) {
            //     $data['boost_cat'] = $boost['category'][$product->id_category_default];
            // }
            // if (isset($boost['manufacturer'][$product->id_manufacturer])) {
            //     $data['boost_manu'] = $boost['manufacturer'][$product->id_manufacturer];
            // }
            // if (isset($boost['supplier'][$product->id_supplier])) {
            //     $data['boost_supp'] = $boost['supplier'][$product->id_supplier];
            // }

            if ($this->exportFeatures) {
                $productFeatures = Product::getFrontFeaturesStatic($this->idLang, $data['id_product']);
                $validFeatures = array();

                if (function_exists('customFeatureAdd')) {
                    $custom_feat = customFeatureAdd($data);
                    if ($custom_feat) {
                        $validFeatures = $custom_feat;
                    }
                }

                $validFeatures[] = $this->setFreeShipping($this->config['sniper_free_ship'], $data['price_showed'])[0];

                foreach ($productFeatures as $productFeature) {
                    if (is_array($this->enabledIdFeatures) &&
                        in_array($productFeature['id_feature'], $this->enabledIdFeatures)
                    ) {
                        $validFeatures[] = $productFeature;
                    }
                }
                unset($productFeatures);
                $data['features'] = Tools::jsonEncode($validFeatures);
            } else {
                $data['features'] = '';
            }

            if ($this->exportAttributes) {
                $productAttributesDetail = Product::getAttributesInformationsByProduct($data['id_product']);
                $validAttributes = array();
                foreach ($productAttributesDetail as $productAttribute) {
                    if (is_array($this->enabledIdAttributeGroups) &&
                        in_array($productAttribute['id_attribute_group'], $this->enabledIdAttributeGroups)
                    ) {
                        $validAttributes[] =
                            array('name' => $productAttribute['group'], 'value' => $productAttribute['attribute']);
                    }
                }

                $validAttributesClean = array_unique($validAttributes, SORT_REGULAR);

                $data['attributes'] = Tools::jsonEncode(array_values($validAttributesClean));
            } else {
                $data['attributes'] = '';
            }

            // tags
            $product_tags = array();
            $presta_tags = Tag::getProductTags((int)$data['id_product']);
            if (!empty($presta_tags) && isset($presta_tags[$this->idLang])) {
                foreach ($presta_tags[$this->idLang] as $tag_data) {
                    $product_tags[] = $tag_data;
                }
            }

            $data['tags'] = Tools::jsonEncode($product_tags);

            if (function_exists('customDataParser')) {
                customDataParser($data);
            }

            $data['alternate_image_url'] = $this->getAlternateImageLink($product, $this->link, $this->config);

            $this->stripTagsFromDesc($data);

            return $data;
        }
    }

    private function stripTagsFromDesc(&$data)
    {
        $data['description_short'] = strip_tags($data['description_short']);
        $data['description']       = strip_tags($data['description']);
        $data['description_short'] = html_entity_decode($data['description_short'], ENT_COMPAT, 'UTF-8');
        $data['description']       = html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8');
        $data['description_short'] = preg_replace("/\r|\n|\t/", " ", $data['description_short']);
        $data['description']       = preg_replace("/\r|\n|\t/", " ", $data['description']);
    }

    private function getImageLink($product, $link, $config)
    {
        $prodImg     = Image::getCover($product->id);
        if ($prodImg === false) {
            return $config['base_url']."modules/sniperfast/views/img/notfound.jpg";
        };
        $imgUrl = $link->getImageLink($product->link_rewrite, $prodImg['id_image'], $config['img_type']);
        $this->setImageHttpProtocol($imgUrl, $config['use_https']);

        return $imgUrl;
    }

    private function getAlternateImageLink($product, $link, $config)
    {
        $sql = "SELECT id_image FROM "._DB_PREFIX_."image ".
                "WHERE cover IS NULL AND id_product = ".(int)$product->id." ORDER BY POSITION ASC LIMIT 1";
        $prodImg     = Db::getInstance()->executeS($sql);

        if (empty($prodImg)) {
            return "";
        };

        $imgUrl = $link->getImageLink($product->link_rewrite, $prodImg[0]['id_image'], $config['img_type']);
        $this->setImageHttpProtocol($imgUrl, $config['use_https']);

        return $imgUrl;
    }

    private function setImageHttpProtocol(&$imgUrl, $useHttpsForImages)
    {
        if (Tools::strpos($imgUrl, 'https://') !== false && !$useHttpsForImages) {
            $imgUrl = str_replace('https://', 'http://', $imgUrl);
        } elseif (Tools::strpos($imgUrl, 'http://') !== false && $useHttpsForImages) {
            $imgUrl = str_replace('http://', 'https://', $imgUrl);
        }
        if (Tools::strpos($imgUrl, 'http') === false || Tools::strpos($imgUrl, 'http') > 0) {
            if ($useHttpsForImages) {
                $imgUrl = 'https://'.$imgUrl;
            } else {
                $imgUrl = 'http://'.$imgUrl;
            }
        }
    }

    private function setFreeShipping($freeShippingLimit, $productPrice)
    {
        $tmp = array();
        if (!is_numeric($freeShippingLimit) || $freeShippingLimit <= 0) {
            return $tmp;
        }

        /* Add free shipping */
        if ($productPrice >= $freeShippingLimit) {
            $tmp[] = array('id_feature' => 9997, 'name' => 'snp_free_shipping', 'value' => 1, 'position' => 0);
        } else {
            $tmp[] = array('id_feature' => 9997, 'name' => 'snp_free_shipping', 'value' => 0, 'position' => 0);
        }

        return $tmp;
    }

    public function sendData($productsToSend)
    {
        $prod = Tools::jsonEncode($productsToSend);
        // file_put_contents(dirname(__FILE__).'/single_prod.json', $prod);
        // return;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, CDN_PROTOCOL.'://index.sniperfast.'.CDN_ENDPOINT.'/schedule');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(array(
            'shop'  => $this->idShop,
            'lang'  => $this->idLang,
            'key'   => SniperCommon::getConfig('sniper_key', $this->idShop, $this->idLang),
            'index' => SniperCommon::getConfig('sniper_index_name', $this->idShop, $this->idLang),
            'ip'    => $_SERVER['SERVER_ADDR'],
            'products' => $prod
        )));

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $res = curl_exec($ch);
        curl_close($ch);
        $res = Tools::jsonDecode($res, true);

        if (isset($res['status']) && $res['status'] == 'ok') {
            return 1;
        }
        return 0;
    }
}
